import java.awt.*;
import java.awt.image.BufferedImageOp;
import java.awt.image.BufferedImage;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.io.*;
import javax.imageio.ImageIO;
import javax.swing.*;

public class MyImage extends JPanel {
    private BufferedImage image;
    private Dimension size = new Dimension();

    public MyImage(String path) {
        try{
            this.image = ImageIO.read(new File(path));
            size.setSize(image.getWidth(), image.getHeight());
        }
        catch(Exception e)
        {
            System.out.println("Error creating image with path "+path);
            System.out.println(e);
        }            
    }
    
    public MyImage(MyImage i)
    {
        size=new Dimension(i.size);
        image = i.image.getSubimage(0,0,getWidth(),getHeight());
    }
    
    /**
     * Drawing an image can allow for more
     * flexibility in processing/editing.
     */
    protected void paintComponent(Graphics g) {
        // Center image in this component.
        int x = (getWidth() - size.width)/2;
        int y = (getHeight() - size.height)/2;
        g.drawImage(image, x, y, this);
    }

    public void display(){
        try{
            JFrame f = new JFrame();
            f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            f.add(new JScrollPane(this));
            f.setSize(image.getWidth(),image.getHeight());
            f.setLocation(10,10);
            f.setVisible(true);
        }
        catch(Exception e){}
    }
    
    public void paintUnder(MyImage otherimage, int decalx, int decaly)
    {
        BufferedImage newImage=otherimage.image;
        
        int sizex = newImage.getWidth();
        int sizey = newImage.getHeight();
        
        int pixel;
        for(int x=0; x<sizex; x++)
            for(int y=0; y<sizey; y++)
            {
                pixel = newImage.getRGB(x,y);
                int alpha_new = (pixel >>24) & 0xFF;
                if( ((x+decalx) < image.getWidth()) &&
                    ((y+decaly) < image.getHeight()) &&
                    ((x+decalx) >= 0 ) &&
                    ((y+decaly) >= 0 ) )
                {
                    int pixel_orig = image.getRGB(x+decalx,y+decaly);
                    int alpha_orig = ((pixel_orig >>24) & 0xFF);
                    if(alpha_orig<255)
                    {
                        //image.setRGB(x+decalx,y+decaly,65536*256*255);
                        double alpha = alpha_orig/255.0;    
                        int red_orig = ((pixel_orig >>16) & 0xFF);
                        int green_orig = ((pixel_orig >>8 ) & 0xFF);
                        int blue_orig = (pixel_orig & 0xFF);
                        int red_new = ((pixel >>16 ) & 0xFF);
                        int green_new = ((pixel >>8 ) & 0xFF);
                        int blue_new = (pixel & 0xFF);
                        int final_red = (int)(red_orig*alpha + red_new*(1.0-alpha));
                        int final_green = (int)(green_orig*alpha + green_new*(1.0-alpha));
                        int final_blue = (int)(blue_orig*alpha + blue_new*(1.0-alpha));
                        int alpha_final = (int)((1.0 - (1.0-alpha)*(1.0-alpha_new/255.0))*255.0);
                        int pixel_final = 65536*256*alpha_final+final_red*65536+final_green*256+final_blue;
                        image.setRGB(x+decalx,y+decaly,pixel_final);
                    }
                }
            }
    }
    
    public void paintOver(MyImage otherimage, int decalx, int decaly)
    {
        BufferedImage newImage=otherimage.image;
        
        int sizex = newImage.getWidth();
        int sizey = newImage.getHeight();
        
        int pixel;
        for(int x=0; x<sizex; x++)
            for(int y=0; y<sizey; y++)
            {
                pixel = newImage.getRGB(x,y);
                int alpha_new = (pixel >>24) & 0xFF;
                if(alpha_new>0)
                {
                    if( ((x+decalx) < image.getWidth()) &&
                        ((y+decaly) < image.getHeight()) &&
                        ((x+decalx) >= 0 ) &&
                        ((y+decaly) >= 0 ) )
                    {
                        double alpha = alpha_new/255.0;
                        int pixel_orig = image.getRGB(x+decalx,y+decaly);
                        int alpha_orig = ((pixel_orig >>16) & 0xFF);
                        int red_orig = ((pixel_orig >>16) & 0xFF);
                        int green_orig = ((pixel_orig >>8 ) & 0xFF);
                        int blue_orig = (pixel_orig & 0xFF);
                        int red_new = ((pixel >>16 ) & 0xFF);
                        int green_new = ((pixel >>8 ) & 0xFF);
                        int blue_new = (pixel & 0xFF);
                        int final_red = (int)(red_new*alpha + red_orig*(1.0-alpha));
                        int final_green = (int)(green_new*alpha + green_orig*(1.0-alpha));
                        int final_blue = (int)(blue_new*alpha + blue_orig*(1.0-alpha));
                        int alpha_final = (int)((1.0 - (1.0-alpha)*(1.0-alpha_orig/255.0))*255.0);
                        int pixel_final = 65536*256*alpha_final+final_red*65536+final_green*256+final_blue;
                        image.setRGB(x+decalx,y+decaly,pixel_final);
                    }
                }
            }
    }
    
    
    public void pixelate(int factor)
    {
        BufferedImageOp op = new AffineTransformOp( AffineTransform.getScaleInstance(1.0/factor, 1.0/factor),
                                                    new RenderingHints(RenderingHints.KEY_INTERPOLATION,
                                                    RenderingHints.VALUE_INTERPOLATION_BICUBIC));
        
        BufferedImage small = op.filter(image, null);
        
        int sizex = image.getWidth();
        int sizey = image.getHeight();
        
        int pixel;
        for(int x=0; x<sizex; x++)
            for(int y=0; y<sizey; y++)
            {
                pixel = small.getRGB(x/factor,y/factor);
                image.setRGB(x,y,pixel);
            }        
    }
    
    
    public void blackandwhite()
    {
        int r,g,b,alpha;
        int sizex = image.getWidth();
        int sizey = image.getHeight();
        
        int pixel;
        for(int x=0; x<sizex; x++)
            for(int y=0; y<sizey; y++)
            {
                pixel = image.getRGB(x,y);
                alpha = ((pixel>>24) & 0xFF);
                r = ((pixel >>16 ) & 0xFF);
                g = ((pixel >>8 ) & 0xFF);
                b = (pixel & 0xFF);
                r=(r+g+b)/3;
                pixel=alpha*256*65536+r*65536+r*256+r;
                image.setRGB(x,y,pixel);
            }        
    }
    
    public void fourbitscolor()
    {
        int r,g,b,alpha;
        int sizex = image.getWidth();
        int sizey = image.getHeight();
        
        int pixel;
        for(int x=0; x<sizex; x++)
            for(int y=0; y<sizey; y++)
            {
                pixel = image.getRGB(x,y);
                alpha = ((pixel>>24) & 0xFF);
                r = ((pixel >>16 ) & 0xFF);
                g = ((pixel >>8 ) & 0xFF);
                b = (pixel & 0xFF);
                r = r - r%16;
                g = g - g%16;
                b = b - b%16;
                pixel=65536*256*alpha+r*65536+g*256+b;
                image.setRGB(x,y,pixel);
            }        
    }
    
    
                
   
}